#######################################################################################################################################################################
#
# Add WMI filter Windows 10 & Windows Server 2016
#
#######################################################################################################################################################################
Function Set-DCAllowSystemOnlyChange
{
	param ([switch]$Set)
	if ($Set)
	{
		Write-Verbose "Checking is registry key is set to allow changes to AD System Only Attributes is set."
		$ntds_vals = (Get-Item HKLM:\System\CurrentControlSet\Services\NTDS\Parameters).GetValueNames()
		if ( $ntds_vals -eq "Allow System Only Change")
		{
			$kval = Get-ItemProperty HKLM:\System\CurrentControlSet\Services\NTDS\Parameters -name "Allow System Only Change"
			if ($kval -eq "1")
			{
		    	Write-Verbose "Allow System Only Change key is already set"    
			}
			else
			{
		    	Write-Verbose "Allow System Only Change key is not set"
				Write-Verbose "Creating key and setting value to 1"
				Set-ItemProperty HKLM:\System\CurrentControlSet\Services\NTDS\Parameters -name "Allow System Only Change" -Value 0 | Out-Null
			}
		}
		else
		{
			New-ItemProperty HKLM:\System\CurrentControlSet\Services\NTDS\Parameters -name "Allow System Only Change" -Value 1 -PropertyType "DWord" | Out-Null
		}
	}
	else
	{
		$ntds_vals = (Get-Item HKLM:\System\CurrentControlSet\Services\NTDS\Parameters).GetValueNames()
		if ( $ntds_vals -eq "Allow System Only Change")
		{
			Write-Verbose "Disabling Allow System Only Change Attributes on server"
			Set-ItemProperty HKLM:\System\CurrentControlSet\Services\NTDS\Parameters -name "Allow System Only Change" -Value 0 | Out-Null
		}
	}
}
#######################################################################################################################################################################
Function Create-WMIFilter
{
 param ($WMIFilterName,$WMIFilterDesc,$WMIFilterReq)
 # Based on function from http://gallery.technet.microsoft.com/scriptcenter/f1491111-9f5d-4c83-b436-537eca9e8d94
 $defaultNamingContext=(get-adrootdse).defaultnamingcontext 
 $configurationNamingContext=(get-adrootdse).configurationNamingContext 
 $msWMIAuthor="Administrator@"+[System.DirectoryServices.ActiveDirectory.Domain]::getcurrentdomain().name
 Write-Host "Starting creation of WMI Filter"
 $WMIGUID=[string]"{"+([System.Guid]::NewGuid())+"}"   
 $WMIDN="CN="+$WMIGUID+",CN=SOM,CN=WMIPolicy,CN=System,"+$defaultNamingContext
 $WMICN=$WMIGUID
 $WMIdistinguishedname=$WMIDN
 $WMIID=$WMIGUID
 $now=(Get-Date).ToUniversalTime()
 $msWMICreationDate=($now.Year).ToString("0000")+($now.Month).ToString("00")+($now.Day).ToString("00")+($now.Hour).ToString("00")+
                    ($now.Minute).ToString("00")+($now.Second).ToString("00")+"."+($now.Millisecond * 1000).ToString("000000")+"-000"
 $msWMIName=$WMIFilterName
 $msWMIParm1=$WMIFilterDesc
 $msWMIParm2="1;3;10;"+$WMIFilterReq.Length.ToString()+";WQL;root\CIMv2;"+$WMIFilterReq+";"
 $Attr=@{"msWMI-Name" = $msWMIName;
         "msWMI-Parm1" = $msWMIParm1;
         "msWMI-Parm2" = $msWMIParm2;
         "msWMI-Author" = $msWMIAuthor;
         "msWMI-ID" = $WMIID;
         "instanceType" = 4;
         "showInAdvancedViewOnly" = "TRUE";
         "distinguishedname" = $WMIdistinguishedname;
         "msWMI-ChangeDate" = $msWMICreationDate; 
         "msWMI-CreationDate" = $msWMICreationDate}
 $WMIPath=("CN=SOM,CN=WMIPolicy,CN=System,"+$defaultNamingContext)
 Write-Host "Adding WMI Filter : $msWMIName"
 New-ADObject -Name $WMICN -Type "msWMI-Som" -Path $WMIPath -OtherAttributes $Attr | Out-Null
 Write-Host "Finished !"
}
#######################################################################################################################################################################
Import-Module ActiveDirectory
Set-DCAllowSystemOnlyChange -Set
Create-WMIFilter "Windows 10 & Windows Server 2016" "Detected OS : Windows 10 & Windows Server 2016" "SELECT * FROM Win32_OperatingSystem WHERE Version LIKE ""10.0%""" 
Set-DCAllowSystemOnlyChange 
#######################################################################################################################################################################
